# -*- coding: utf-8 -*-
# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Convenience tools to manage drivers."""

import logging
from typing import Any, Optional

from invoke import Runner  # Executor

logging.getLogger(__name__).addHandler(logging.NullHandler())


class Exec(Runner):
    """Provide Invoke integration with Stevedore."""

    def __init__(self, start: Optional[str] = None, **kwargs: Any) -> None:
        """Initialize collection."""
        self.plugins = kwargs['config']['plugins']
        super().__init__(**kwargs)
        self.config = kwargs.pop('config', {})
        if start is None:
            start = self.config.tasks.search_root
        self._start = start
        # namespace=kwargs.pop('driver_namespace'),
        # name=kwargs.pop('driver_name', name),
        # **kwargs,



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Static Content Task-Runner."""

from typing import Optional

from invoke import Context, task


@task
def install(
    ctx, path=None, also='dev'
):  # type: (Context, Optional[str], str) -> None
    """Install WebUI dependencies."""
    with ctx.cd(path or ctx.webui_dir):
        ctx.run(f"npm install --also={also}")


@task(pre=[install])
def build(ctx, path=None):  # type: (Context, Optional[str]) -> None
    """Build static content."""
    with ctx.cd(path or ctx.webui_dir):
        ctx.run('npm run build')


@task
def start(
    ctx, host=None, port=8000, browser=False, path=None
):  # type: (Context, Optional[str], int, bool, Optional[str]) -> None
    """Start webui development."""
    args = []
    if browser:
        args.append('--open')
    if host:
        args.append(f"--host={host}")
    if port:
        args.append(f"--port={port}")
    with ctx.cd(path or ctx.webui_dir):
        print(f"npx vue-cli-service serve {' '.join(args)}")
        ctx.run(
            'npx vue-cli-service serve',  # {}".format(' '.join(args)),
            asynchronous=True,
        )


@task
def stop(ctx):  # type: (Context) -> None
    """Stop WebUI development."""
    ctx.run('pkill npm')



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Run system tests using test-infra."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(
    ctx,  # type: Context
):  # type: (...) -> None
    """Run test-infra tests."""
    print('test-infra stub')


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(
    ctx,  # type: Context
    marker=None,  # type: str
    capture=None,  # type: Optional[str]
    debugger=None,  # type: Optional[str]
    durations=None,  # type: Optional[int]
    durations_min=None,  # type: Optional[int]
    maxfail=None,  # type: Optional[int]
):  # type: (...) -> None
    """Perform unit tests."""
    from pprint import pprint
    pprint(ctx.config.__dict__)
    args = []
    if marker:
        args.append(f"-m {marker}")
    if capture:
        args.append(f"--capture={capture}")
    if debugger:
        args.append('--pdb')
    if durations:
        args.append(f"--durations={durations}")
    if durations_min:
        args.append(f"--durations-min={durations_min}")
    if maxfail:
        args.append(f"--maxfail={maxfail}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"pytest {' '.join(args)}")


@task
def coverage(
    ctx,  # type: Context
    # project_dir,  # type: str
    coverage=True,  # type: bool
    coverage_on_fail=True,  # type: bool
    minimum=80,  # type: int
    append=False,  # type: bool
    branch=None,  # type: Optional[str]
    context=None,  # type: Optional[str]
    configpath=None,  # type: Optional[str]
    report=None,  # type: Optional[str]
):  # type: (...) -> None
    """Perform coverage checks for tests."""
    args = []
    if coverage:
        args.append('--no-cov')
    if not coverage_on_fail:
        args.append('--no-cov-on-fail')
    else:
        # args.append(f"--cov={project_dir}")
        if configpath:
            args.append(f"--cov-config={configpath}")
        if minimum:
            args.append(f"--cov-fail-under={minimum}")
        if append:
            args.append('--cov-append')
        if branch:
            args.append('--cov-branch')
        if context:
            args.append(f"--cov-context={context}")
        if report:
            args.append(f"--cov-report={report}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"pytest {' '.join(args)}")


namespace = Collection(coverage, run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide setuptools packaging task-runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context

# TODO: handle setup() here?


@task
def build(
    ctx,
    kind='wheel',
    outdir=None,
    skip_dependency_check=False,
    no_isolation=False,
):  # type: (Context, str, Optional[str], bool, bool) -> None
    """Build package."""
    args = []
    if kind == 'wheel':
        args.append('--wheel')
    elif kind == 'sdist':
        args.append('--sdist')
    else:
        raise Exception('cannot build unknown package format')
    if outdir:
        args.append(f"--outdir={outdir}")
    if skip_dependency_check:
        args.append('--skip-dependency-check')
    if no_isolation:
        args.append('--no-isolation')
    ctx.run(f"{ctx.python_path} -m build {' '.join(args)}")


tasks = Collection(build)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Build packages using flit."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(
    ctx, kind=None, no_setup_py=True
):  # type: (Context, Optional[bool], bool) -> None
    """Build wheel package."""
    args = []
    if kind:
        args.append(f"--format={kind}")
    if no_setup_py:
        args.append('--no-setup-py')
    ctx.run(f"flit build {' '.join(args)}")


@task
def install(
    ctx,  # type: Context
    kind='symlink',  # type: str
    user=None,  # type: Optional[bool]
    env=None,  # type: Optional[bool]
    deps=None,  # type: Optional[str]
    extras=None,  # type: Optional[str]
):  # type: (...) -> None
    """Install within environment."""
    args = []
    if kind == 'symlink':
        args.append('--symlink')
    elif kind == 'pth-file':
        args.append('--pth-file')
    if user is not None:
        args.append(f"--user={user}")
    if env is not None:
        args.append(f"--env={env}")
    if deps:
        args.append(f"--deps={deps}")
    if extras:
        args.append(f"--extras={extras}")
    ctx.run(f"flit install {' '.join(args)}")


@task
def publish(
    ctx,
    kind=None,
    no_setup_py=True,
    repository=None,
):  # type: (Context, Optional[str], bool, Optional[str]) -> None
    """Publish project distribution."""
    args = []
    if kind:
        args.append(f"--format={kind}")
    if no_setup_py:
        args.append('--no-setup-py')
    if repository:
        args.append(f"--repository={repository}")
    ctx.run(f"flit publish {' '.join(args)}")


tasks = Collection(build, install, publish)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide documentation task-runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(
    ctx,  # type: Context
    clean=False,  # type: bool
    configfile=None,  # type: Optional[str]
    strict=False,  # type: bool
    theme=None,  # type: Optional[str]
    use_directory_urls=None,  # type: Optional[str]
    path=None,  # type: Optional[str]
    quiet=False,  # type: bool
    verbose=False,  # type: bool
):  # type: (...) -> None
    """Build documentation site."""
    args = []
    if clean is not None:
        args.append('--clean' if clean else '--dirty')
    if configfile:
        args.append(f"--config-file {configfile}")
    if strict:
        args.append('--strict')
    if theme:
        if theme in ['readthedocs', 'material', 'mkdocs']:
            args.append(f"--theme {theme}")
        else:
            raise Exception(f"{theme} is not a supported mkdocs theme")
    if use_directory_urls is not None:
        args.append(
            '--use-directory-urls'
            if use_directory_urls
            else '--no-directory-urls'
        )
        args.append(f"--site-dir {path}")
    if quiet:
        args.append('--quiet')
    if verbose:
        args.append('--verbose')

    with ctx.cd(ctx.project_dir):
        ctx.run(f"mkdocs build {' '.join(args)}")


@task
def start(
    ctx,  # type: Context
    address='localhost:8001',  # type: str
    livereload=None,  # type: Optional[bool]
    dirtyreload=None,  # type: Optional[bool]
    watch_theme=False,  # type: bool
    configfile=None,  # type: Optional[str]
    strict=False,  # type: bool
    theme='readthedocs',  # type: str
    use_directory_urls=None,  # type: Optional[bool]
    quiet=False,  # type: bool
    verbose=False,  # type: bool
):  # type: (...) -> None
    """Start docsite."""
    args = [f"--dev-addr {address}"]
    if livereload is not None:
        args.append('--livereload' if livereload else '--no-livereload')
    if dirtyreload:
        args.append('--dirtyreload')
    if watch_theme:
        args.append('--watch-theme')
    if configfile:
        args.append(f"--config-file {configfile}")
    if strict:
        args.append('--strict')
    if theme:
        if theme in ['readthedocs', 'mkdocs', 'material']:
            args.append(f"--theme {theme}")
        else:
            raise Exception('unsupported theme')
    if use_directory_urls is not None:
        args.append(
            '--use-directory-urls'
            if use_directory_urls
            else '--no-directory-urls'
        )
    if quiet:
        args.append('--quiet')
    elif verbose:
        args.append('--verbose')

    with ctx.cd(ctx.docs_dir):
        ctx.run(f"mkdocs serve {' '.join(args)}", disown=True)


@task
def stop(ctx):  # type: (Context) -> None
    """Stop docsite."""
    ctx.run('pkill mkdocs')


namespace = Collection(build, start, stop)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task(name='format')
def style(ctx):  # type: (Context) -> None
    """Format project source code to PEP8 standard."""
    ctx.run(f"autopep8 --in-place --recursive {ctx.project_dir}")


namespace = Collection(style)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Deploy stack using Ansible."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def install(
    ctx,  # type: Context
):  # type: (...) -> None
    """Install Ansible collections or roles."""
    ...


@task
def run(
    ctx,  # type: Context
):  # type: (...) -> None
    """Run Ansible deployment."""
    ...


namespace = Collection(install, run)



# -*- coding: utf-8 -*-
# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Convenience tools to manage drivers."""

import logging
from typing import Any, Dict, List, Optional

from invoke import Collection as InvokeCollection
from invoke.config import merge_dicts

# from stevedore.extension import ExtensionManager
from stevedore.driver import DriverManager

logging.getLogger(__name__).addHandler(logging.NullHandler())


class Collection(InvokeCollection):
    """Provide Invoke integration with Stevedore."""

    # TODO: would pre/post executor be usefull here

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize collection."""
        configuration = kwargs.pop('configuration', {})
        super().__init__(*args, **kwargs)
        self._configuration = merge_dicts(
            self._configuration, configuration  # type: ignore
        )
        if 'plugins' in self._configuration:
            self.load_collections(self._configuration['plugins'])

    @staticmethod
    def _get_driver(
        name: Optional[str] = None, **kwargs: Any
    ) -> 'DriverManager':
        """Get driver for collection."""
        driver_manager = DriverManager(
            namespace=kwargs.pop('driver_namespace'),
            name=kwargs.pop('driver_name', name),
            **kwargs,
        )
        return driver_manager

    @classmethod
    def load_module(
        cls,
        name: str,
        config: Optional[str] = None,
        loaded_from: Optional[str] = None,
        auto_dash_names: Optional[bool] = None,
        **kwargs: Any,
    ) -> 'Collection':
        """Add module dynamically from entrypoint."""
        driver_manager = cls._get_driver(name=name, **kwargs)
        collection = cls.from_module(
            driver_manager.driver,
            name=name,
            config=None,
            loaded_from=None,
            auto_dash_names=None,
        )
        return collection

    def load_collection(
        self,
        name: str,
        default: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Add collection from entrypoint."""
        driver_manager = self._get_driver(name=name, **kwargs)
        self.add_collection(
            driver_manager.driver,
            name=name,
            default=default,
        )

    def load_collections(self, plugins: List[Dict[str, Any]] = []) -> None:
        """Load collections from configuration."""
        for plugin in plugins or self._configuration['plugins']:
            self.load_collection(**plugin)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide App Task-Runner."""

import os
import textwrap
from typing import TYPE_CHECKING

from invoke import task

if TYPE_CHECKING:
    from invoke import Context


@task
def start(
    ctx, hostname='localhost', port=8080, workers=4
):  # type: (Context, str, int, int) -> None
    """Start webapp."""
    with ctx.cd(ctx.webapp_dir):
        ctx.run(
            textwrap.dedent(
                f"""\
                gunicorn app:app \
                --pid={os.getcwd()}/.pid \
                --bind={hostname}:{port} \
                --workers={workers} \
                --reload
            """
            ),
            disown=True,
        )


@task
def stop(ctx):  # type: (Context) -> None
    """Stop webapp."""
    ctx.run('pkill gunicorn')



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide documentation task-runner for Sphinx."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(
    ctx,  # type: Context
):  # type: (...) -> None
    """Build documentation site."""
    pass


@task
def start(
    ctx,  # type: Context
):  # type: (...) -> None
    """Start docsite."""
    pass


@task
def stop(ctx):  # type: (Context) -> None
    """Stop docsite."""
    pass


namespace = Collection(build, start, stop)



# -*- coding: utf-8 -*-
"""Publish packages using Twine."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def check(ctx, path, strict=True):  # type: (Context, str, bool) -> None
    """Check package."""
    args = [path]
    if strict:
        args.append('--strict')
    ctx.run(f"twine check {' '.join(args)}")


@task
def register(
    ctx,  # type: Context
    repository=None,  # type: Optional[str]
    repository_url=None,  # type: Optional[str]
    sign=False,  # type: bool
    sign_with=None,  # type: Optional[str]
    identity=None,  # type: Optional[str]
    username=None,  # type: Optional[str]
    password=None,  # type: Optional[str]
    comment=None,  # type: Optional[str]
    skip_existing=False,  # type: bool
    cert_path=None,  # type: Optional[str]
    client_cert_path=None,  # type: Optional[str]
    verbose=False,  # type: bool
    disable_progress_bar=False,  # type: bool
):  # type: (...) -> None
    """Register package to a PyPI repository."""
    args = ['--non-interactive']
    if repository:
        args.append(f"--repository={repository}")
    if repository_url:
        args.append(f"--repository-url={repository_url}")
    if sign:
        args.append('--sign')
        if sign_with:
            args.append(f"--sign-with='{sign_with}'")
        else:
            raise Exception('need to provide signature for package signing')
    if identity:
        args.append(f"--identity='{identity}'")
    if username:
        args.append(f"--username={username}")
        if password:
            args.append(f"--password='{password}'")
        else:
            raise Exception('username requires a password')
    if comment:
        args.append(f"--comment='{comment}'")
    if skip_existing:
        args.append('--skip-existing')
    if cert_path:
        args.append(f"--cert={cert_path}")
    if client_cert_path:
        args.append(f"--client-cert={client_cert_path}")
    if verbose:
        args.append('--verbose')
    if disable_progress_bar:
        args.append('--disable-progress-bar')
    ctx.run(f"twine register {' '.join(args)}")


@task
def publish(
    ctx,  # type: Context
    repository=None,  # type: Optional[str]
    repository_url=None,  # type: Optional[str]
    sign=False,  # type: bool
    sign_with=None,  # type: Optional[str]
    identity=None,  # type: Optional[str]
    username=None,  # type: Optional[str]
    password=None,  # type: Optional[str]
    comment=None,  # type: Optional[str]
    skip_existing=False,  # type: bool
    cert_path=None,  # type: Optional[str]
    client_cert_path=None,  # type: Optional[str]
    verbose=False,  # type: bool
    disable_progress_bar=False,  # type: bool
):  # type: (...) -> None
    """Upload package to a PyPI repository."""
    args = ['--non-interactive']
    if repository:
        args.append(f"--repository={repository}")
    if repository_url:
        args.append(f"--repository-url={repository_url}")
    if sign:
        args.append('--sign')
        if sign_with:
            args.append(f"--sign-with='{sign_with}'")
        else:
            raise Exception('need to provide signature for package signing')
    if identity:
        args.append(f"--identity='{identity}'")
    if username:
        args.append(f"--username={username}")
        if password:
            args.append(f"--password='{password}'")
        else:
            raise Exception('username requires a password')
    if comment:
        args.append(f"--comment='{comment}'")
    if skip_existing:
        args.append('--skip-existing')
    if cert_path:
        args.append(f"--cert {cert_path}")
    if client_cert_path:
        args.append(f"--client-cert {client_cert_path}")
    if verbose:
        args.append('--verbose')
    if disable_progress_bar:
        args.append('--disable-progress-bar')
    ctx.run(f"twine upload {' '.join(args)}")


namespace = Collection(check, register, publish)



# -*- coding: utf-8 -*-
"""Publish packages using Poetry."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(ctx, kind=None):  # type: (Context, Optional[bool]) -> None
    """Build wheel package."""
    args = ['--no-interaction']
    if kind:
        args.append(f"--format={kind}")
    ctx.run(f"poetry build {' '.join(args)}")


@task
def install(
    ctx,  # type: Context
    dev=True,  # type: bool
    editable=True,  # type: bool
    remove_untracked=False,  # type: bool
    extras=None,  # type: Optional[str]
):  # type: (...) -> None
    """Install project into local environment."""
    args = ['--no-interaction']
    if not dev:
        args.append('--no-dev')
    if not editable:
        args.append('--no-root')
    if remove_untracked:
        args.append('--remove-untracked')
    if extras:
        args.append(f"--extras={extras}")
    ctx.run(f"poetry install {' '.join(args)}")


@task
def publish(
    ctx,  # type: Context
    repository_url=None,  # type: Optional[str]
    username=None,  # type: Optional[str]
    password=None,  # type: Optional[str]
    cert_path=None,  # type: Optional[str]
    client_cert_path=None,  # type: Optional[str]
):  # type: (...) -> None
    """Publish package to a PyPI repository."""
    args = ['--no-interaction']
    # TODO: check if credentials exist run setup otherwise
    if repository_url:
        args.append(f"--repository={repository_url}")
    if username:
        args.append(f"--username={username}")
        if password:
            args.append(f"--password='{password}'")
        else:
            raise Exception('username requires a password')
    if cert_path:
        args.append(f"--cert={cert_path}")
    if client_cert_path:
        args.append(f"--client-cert={client_cert_path}")
    ctx.run(f"poetry publish {' '.join(args)}")


namespace: Collection = Collection(build, install, publish)



# -*- coding: utf-8 -*-
"""Publish packages using Poetry."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(ctx, kind=None):  # type: (Context, Optional[bool]) -> None
    """Build wheel package."""
    args = ['--no-interaction']
    if kind:
        args.append(f"--format={kind}")
    ctx.run(f"poetry build {' '.join(args)}")


# [-h] [-v] [-D] [-F] [--specpath DIR] [-n NAME]
# [--add-data <SRC;DEST or SRC:DEST>]
# [--add-binary <SRC;DEST or SRC:DEST>] [-p DIR]
# [--hidden-import MODULENAME]
# [--collect-submodules MODULENAME]
# [--collect-data MODULENAME] [--collect-binaries MODULENAME]
# [--collect-all MODULENAME] [--copy-metadata PACKAGENAME]
# [--recursive-copy-metadata PACKAGENAME]
# [--additional-hooks-dir HOOKSPATH]
# [--runtime-hook RUNTIME_HOOKS] [--exclude-module EXCLUDES]
# [--key KEY] [--splash IMAGE_FILE]
# [-d {all,imports,bootloader,noarchive}] [-s] [--noupx]
# [--upx-exclude FILE] [-c] [-w]
# [-i <FILE.ico or FILE.exe,ID or FILE.icns or "NONE">]
# [--disable-windowed-traceback] [--version-file FILE]
# [-m <FILE or XML>] [-r RESOURCE] [--uac-admin]
# [--uac-uiaccess] [--win-private-assemblies]
# [--win-no-prefer-redirects]
# [--osx-bundle-identifier BUNDLE_IDENTIFIER]
# [--target-architecture ARCH] [--codesign-identity IDENTITY]
# [--osx-entitlements-file FILENAME] [--runtime-tmpdir PATH]
# [--bootloader-ignore-signals] [--distpath DIR]
# [--workpath WORKPATH] [-y] [--upx-dir UPX_DIR] [-a]
# [--clean] [--log-level LEVEL]
# scriptname [scriptname ...]


namespace: Collection = Collection(build)



"""Provide git schema objecst."""

from typing import Any, List, Tuple

from marshmallow import ValidationError
from marshmallow.fields import Field


class ClientSideHook(Field):
    """Provide validation for client-side git-hooks."""

    hooks: Tuple[str, ...] = (
        'applypatch-msg',
        'pre-applypatch',
        'post-applypatch',
        'pre-commit',
        'pre-merge-commit',
        'prepare-commit-msg',
        'post-commit',
        'pre-rebase',
        'post-checkout',
        'post-merge',
        'pre-push',
        'post-update',
        'reference-transaction',
        'push-to-checkout',
        'pre-auto-gc',
        'post-rewrite',
        'rebase',
        'sendemail-validate',
        'fsmonitor-watchman',
        'p4-changelist',
        'p4-prepare-changelist',
        'p4-post-changelist',
        'p4-pre-submit',
        'post-index-change',
    )

    def _serialize(
        self, value: str, attr: str, obj: object, **kwargs: Any
    ) -> str:
        if value is None:
            return ''
        return ''.join(str(d) for d in value)

    def _deserialize(
        self, value: str, attr: str, data: Any, **kwargs: Any
    ) -> List[int]:
        try:
            return [int(c) for c in value]
        except ValueError as error:
            raise ValidationError(
                'Pin codes must contain only digits.'
            ) from error


class ServerSideHook(Field):
    """Provide validation for client-side git-hooks."""

    hooks: Tuple[str, ...] = (
        'pre-recieve',
        'post-recieve',
        'update',
    )

    def _serialize(
        self, value: str, attr: str, obj: object, **kwargs: Any
    ) -> str:
        if value is None:
            return ''
        return ''.join(str(d) for d in value)

    def _deserialize(
        self, value: str, attr: str, data: Any, **kwargs: Any
    ) -> List[int]:
        try:
            return [int(c) for c in value]
        except ValueError as error:
            raise ValidationError(
                'Pin codes must contain only digits.'
            ) from error



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Validation Task-Runner."""

# import os
# import re

from cerberus import Validator
from invoke import Context, task

# path_check = re.compile('[a-zA-Z0-9_-/.]+')


@task
def check(ctx, path='static && echo unsafe'):  # type: (Context, str) -> None
    """Check to develop cerberus."""
    schema = {'path': {'type': 'string', 'regex': '^$'}}
    v = Validator(schema)
    print(v.validate({'path': path}))
    ctx.run(f"ls {path}")



# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide convenience tool to manage projects with Python."""

import logging
import os
from dataclasses import asdict
from pprint import pprint  # noqa
from typing import List

from invoke import Program
from proman_common.config import Config
from proman_common.filepaths import AppDirs

from proman_workflows import (
    container,
    docs,
    exception,
    package,
    qa,
    sca,
    stlc,
    utils,
)
from proman_workflows.collection import Collection
from proman_workflows.config import (  # WorkflowConfig,
    DocsConfig,
    ProjectConfig,
    ProjectDirs,
)

__author__ = 'Jesse P. Johnson'
__author_email__ = 'jpj6652@gmail.com'
__title__ = 'proman_workflows'
__description__ = 'Convenience module to manage project tools with Python.'
__version__ = '0.1.0a3'
__license__ = 'MPL-2.0'
__copyright__ = 'Copyright 2021 Jesse Johnson.'
__all__ = ['workflow', 'workflow_tools']

logging.getLogger(__name__).addHandler(logging.NullHandler())


class WorkflowProgram(Program):
    """Provide workflow integration with invoke program."""

    def parse_collection(self) -> None:
        """Patch collection parse to load project config."""
        if self.namespace is not None:
            self.config.set_project_location(os.getcwd())
            self.config.load_project()
        super().parse_collection()


def get_specfile(project_dir: str, specfiles: List[str]) -> Config:
    """Get source tree from path."""
    for specfile in specfiles:
        filepath = os.path.join(project_dir, specfile)
        if os.path.isfile(filepath):
            config = Config(filepath=filepath)
            config.combine(config.retrieve('.tool.proman.workflows'))
            return config
    raise exception.PromanWorkflowException('no configuration found')


app_dirs = AppDirs(project_name='proman_workflows')
project_dirs = ProjectDirs()
specfile = get_specfile(
    project_dir=project_dirs.project_dir,
    specfiles=project_dirs.specfiles,
)
project_config = ProjectConfig(
    docs=asdict(DocsConfig()),
    specfile=specfile.data,
    # plugins=specfile.retrieve('.tool.proman.workflows.plugins') or [],
)
# pprint(asdict(project_config))

workflow_namespace = Collection()
# workflow_namespace.configure(
#     {
#         'dirs': asdict(dirs),
#         'spec': specfile.data,
#         'docs': asdict(docs_config),
#         'working_dir': config.working_dir,
#         'container_runtime': config.container_runtime,
#     }
# )
workflow_namespace.add_collection(container.namespace, name='container')
workflow_namespace.add_collection(docs.namespace, name='docs')
workflow_namespace.add_collection(package.namespace, name='dist')
workflow_namespace.add_collection(qa.namespace, name='qa')
workflow_namespace.add_collection(sca.namespace, name='sca')
workflow_namespace.add_collection(utils.namespace, name='utils')
workflow_tools = Program(
    version=__version__,
    namespace=workflow_namespace,
    name=__title__,
    binary='workflow-tools',
    binary_names=['workflow-tools'],
)

workflow_namespace = Collection().from_module(stlc)
workflow_namespace.configure(
    {
        **asdict(app_dirs),
        **specfile,
        # **asdict(project_config),
    }
)
# workflow_namespace.load_collections(plugins=asdict(project_config)['plugins'])
# workflow_namespace.add_collection(init.namespace, name='init')
workflow = WorkflowProgram(
    name=__title__,
    namespace=workflow_namespace,
    binary='workflow',
    binary_names=['workflow'],
    # config_class=WorkflowConfig,
    version=__version__,
)



# copyright: (c) 2020 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide CLI for git-tools."""

import os
import shutil
import sys
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from compendium.loader import ConfigFile
from invoke.config import Config as InvokeConfig

# from invoke.config import merge_dicts
from pygit2 import discover_repository

# from typing import Optional, Tuple
# from urllib.parse import urljoin, urlparse


VENV_PATH = os.getenv('VIRTUAL_ENV', None)
PATHS = [VENV_PATH] if VENV_PATH else []


class WorkflowConfig(InvokeConfig):
    """Provide project configuration."""

    prefix = 'workflow'

    # @staticmethod
    # def global_defaults() -> Dict[str, Any]:
    #     """Set global defaults."""
    #     defaults = InvokeConfig.global_defaults()
    #     default_plugins = {
    #         'plugins': [
    #             {
    #                 'name': 'package',
    #                 'driver_name': 'poetry',
    #                 'driver_namespace': 'proman.workflows.package',
    #             },
    #             # [tool.poetry.plugins."proman.workflows.vcs"]
    #             # git = "proman_workflows.git:namespace"
    #             {
    #                 'name': 'vcs',
    #                 'driver_name': 'git',
    #                 'driver_namespace': 'proman.workflows.vcs',
    #             },
    #             {
    #                 'name': 'sort-headers',
    #                 'driver_name': 'isort',
    #                 'driver_namespace': 'proman.workflows.formatter',
    #             },
    #             {
    #                 'name': 'format',
    #                 'driver_name': 'black',
    #                 'driver_namespace': 'proman.workflows.formatter',
    #             },
    #             {
    #                 'name': 'unit-tests',
    #                 'driver_name': 'pytest',
    #                 'driver_namespace': 'proman.workflows.unit_tests',
    #             },
    #             {
    #                 'name': 'gpg',
    #                 'driver_name': 'gpg',
    #                 'driver_namespace': 'proman.workflows.pki',
    #             },
    #             {
    #                 'name': 'tls',
    #                 'driver_name': 'tls',
    #                 'driver_namespace': 'proman.workflows.pki',
    #             },
    #         ]
    #     }
    #     return merge_dicts(defaults, default_plugins)


@dataclass
class ProjectDirs:
    """Project package."""

    working_dir: str = os.getcwd()
    python_path: str = sys.executable
    repo_dir: str = discover_repository(os.getcwd())
    base_dir: str = os.path.abspath(os.path.join(repo_dir, os.pardir))
    project_dir: str = field(init=False)
    hooks_dir: str = field(init=False)
    templates_dir: str = field(init=False)
    container_build_dir: str = field(init=False)
    specfiles: List[str] = field(default_factory=list, repr=False)

    def __post_init__(self) -> None:
        """Initialize project settings."""
        if self.specfiles == []:
            self.specfiles = ['pyproject.toml', 'setup.cfg']
        if not hasattr(self, 'project_dir'):
            self.project_dir = os.path.abspath(
                os.path.join(self.repo_dir, os.pardir)
            )
        if not hasattr(self, 'hooks_dir'):
            self.hooks_dir = os.path.join(self.repo_dir, 'hooks')
        if not hasattr(self, 'templates_dir'):
            self.templates_dir = os.path.join(self.base_dir, 'templates')
        if not hasattr(self, 'container_build_dir'):
            self.container_build_dir = self.base_dir


@dataclass
class GlobalConfig(ConfigFile):
    """Configuration for project management."""

    name: str
    writable: bool = True
    directory: str = field(init=False)
    filepath: str = field(init=False)

    def __post_init__(self) -> None:
        """Initialize settings from configuration."""
        self.directory = os.path.join(
            os.path.expanduser('~'), '.config', self.name
        )
        self.filepath = os.path.join(self.directory, 'proman.toml')
        super().__init__(
            self.filepath,
            writable=self.writable,
            separator='.',
        )
        if os.path.exists(self.filepath) and os.path.isfile(self.filepath):
            self.load()
        elif not os.path.exists(self.directory):
            os.makedirs(self.directory)
            self.dump()


# @dataclass
# class Credentials:
#     """Manage credentials."""
#
#     username: Optional[str] = None
#     password: Optional[str] = field(default=None, repr=False)
#     interactive: bool = True
#
#     def __post_init__(self) -> None:
#         """Initialize GPG settings."""
#         if self.interactive:
#             if not self.username:
#                 self.username = pyip.inputStr(
#                     prompt='Enter username: ',
#                     limit=255,
#                 )
#             if not self.password:
#                 self.password = pyip.inputPassword(
#                     prompt='Enter password: ',
#                     limit=255,
#                 )


@dataclass
class GPGConfig:
    """Manage GPG keys."""

    # commit_signing_key: str
    # package_signing_key: str
    home_dir: str = os.path.join(os.path.expanduser('~'), '.gnupg')


@dataclass
class DocsConfig:
    """Manage documentation configuration."""

    def __post_init__(self) -> None:
        """Initialize docs path."""
        if hasattr(self, 'base_dir'):
            print(self.base_dir)  # type: ignore


@dataclass
class WebConfig(ProjectDirs):
    """Manage web configuration."""

    environment = os.getenv('FLASK_ENV', 'development')
    webapp_dir: str = field(init=False)
    static_dir: str = field(init=False)

    def __post_init__(self) -> None:
        """Initialize settings."""
        if not hasattr(self, 'webapp_dir'):
            self.webapp_dir: str = os.path.join(self.base_dir, 'ui')
        if not hasattr(self, 'static_dir'):
            self.static_dir: str = os.path.join(self.webapp_dir, 'static')


@dataclass
class Container:
    """Manage container configuration."""

    runtime: Optional[str] = None

    def __post_init__(self) -> None:
        """Initialize container config."""
        if not hasattr(self, 'runtime'):
            if shutil.which('podman'):
                self.runtime = 'podman'
            elif shutil.which('docker'):
                self.runtime = 'docker'


@dataclass
class Plugin:
    """Provide proman_workflows plugin."""

    name: str
    driver_name: str
    driver_namespace: str


@dataclass
class Job:
    """Provide proman_workflows plugin."""

    command: str
    args: Dict[str, Any] = field(default_factory=dict, repr=False)


@dataclass
class Phase:
    """Provide proman_workflows plugin."""

    name: str
    plugins: Optional[List[str]] = field(default=None)
    jobs: List[Job] = field(default_factory=list, repr=False)


@dataclass
class ProjectConfig(ProjectDirs):
    """Configure project settings."""

    docs: Dict[str, Any] = field(default_factory=dict)
    specfile: Dict[str, Any] = field(default_factory=dict)
    plugins: List[Plugin] = field(default_factory=list)
    phases: List[Phase] = field(default_factory=list)



# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Implement software testing life-cycle."""
# TODO: refactor to dynamically populate tasks

import logging
import os
from dataclasses import asdict
from pprint import pprint
from typing import TYPE_CHECKING

from invoke import Executor, task

# TODO: switch to executor
from proman_workflows import formatter, package, qa, sca, system
from proman_workflows.collection import Collection
from proman_workflows.config import (  # Plugin,
    Job,
    Phase,
    ProjectConfig,
    WorkflowConfig,
)

if TYPE_CHECKING:
    from invoke import Context

logging.getLogger(__name__).addHandler(logging.NullHandler())


def execute(
    ctx, collection, task_name
):  # type: (Context, Collection, str) -> None
    """Execute tasks within collection."""
    for x in ctx.phases:
        print(x)
    config = [p for p in ctx.phases if p['name'] == task_name][0]
    jobs = [(j['command'], j['args']) for j in config['jobs']]
    try:
        Executor(
            collection,
            config=WorkflowConfig(ctx.config),
        ).execute(*jobs)
    except Exception as err:
        print(err)


@task
def static_analysis(ctx):  # type: (Context) -> None
    """Perform static code analysis."""
    if 'plugins' in ctx:
        pprint(ctx.plugins)
    else:
        pprint(ctx.config.__dict__)
    print('task names', sca.namespace.task_names)
    # Executor(
    #     sca.namespace,
    #     config=WorkflowConfig(ctx.config)
    # ).execute('package.publish')
    execute(ctx, collection=sca.namespace, task_name='static-analysis')


@task(name='style')
def formatting(ctx):  # type: (Context) -> None
    """Perform install."""
    print('tasks', formatter.namespace.task_names)
    execute(ctx, collection=formatter.namespace, task_name='formatter')


@task
def install(ctx):  # type: (Context) -> None
    """Perform install."""
    execute(ctx, collection=package.namespace, task_name='install')


@task
def build(ctx):  # type: (Context) -> None
    """Perform build."""
    execute(ctx, collection=package.namespace, task_name='build')


@task
def unit_test(ctx):  # type: (Context) -> None
    """Perform unit-testing."""
    print(qa.namespace.task_names)
    # pprint(ctx.config.__dict__['_defaults']['plugins'])
    ctx.project_dir = os.path.join(os.getcwd(), 'tests')
    execute(ctx, collection=qa.namespace, task_name='unit-test')


@task
def integration_test(ctx):  # type: (Context) -> None
    """Perform integration testing."""
    print(qa.namespace.task_names)
    ctx.project_dir = os.path.join(os.getcwd(), 'integration')
    execute(ctx, collection=qa.namespace, task_name='integration-test')


@task
def system_test(ctx):  # type: (Context) -> None
    """Perform system testing."""
    print(system.namespace.task_names)
    execute(ctx, collection=system.namespace, task_name='system-test')


@task
def acceptance_test(ctx):  # type: (Context) -> None
    """Perform acceptance testing."""
    execute(ctx, collection=qa.namespace, task_name='acceptance-test')


@task
def publish(ctx):  # type: (Context) -> None
    """Publish package."""
    print(package.namespace.task_names)
    Executor(
        package.namespace,
        config=WorkflowConfig(ctx.config),
    ).execute('package.publish')


project_config = ProjectConfig(
    # plugins=[
    #     Plugin(
    #         name='poetry',
    #         driver_name='package',
    #         driver_namespace='proman_workflows.package.poetry',
    #     ),
    #     Plugin(
    #         name='vcs',
    #         driver_name='git',
    #         driver_namespace='proman.workflows.vcs',
    #     ),
    #     Plugin(
    #         name='sort-headers',
    #         driver_name='isort',
    #         driver_namespace='proman.workflows.formatter',
    #     ),
    #     Plugin(
    #         name='format',
    #         driver_name='black',
    #         driver_namespace='proman.workflows.formatter',
    #     ),
    #     Plugin(
    #         name='unit-tests',
    #         driver_name='pytest',
    #         driver_namespace='proman.workflows.unit_tests',
    #     ),
    # ],
    phases=[
        Phase(
            name='install',
            plugins=['poetry'],
            jobs=[Job(command='poetry.install', args={})],
        ),
        Phase(
            name='formatter',
            jobs=[
                Job(command='sort-headers.run', args={}),
                Job(command='format.run', args={}),
            ],
        ),
        Phase(
            name='static-analysis',
            jobs=[
                Job(command='lint.run', args={}),
                Job(command='type-checking.run', args={}),
            ],
        ),
        Phase(
            name='build',
            plugins=['poetry'],
            jobs=[Job(command='poetry.build', args={})],
        ),
        Phase(
            name='unit-test',
            jobs=[Job(command='unit-test.run', args={})],
        ),
        Phase(
            name='acceptance-test',
            jobs=[Job(command='acceptance-test.run', args={})],
        ),
        Phase(
            name='integration-test',
            jobs=[Job(command='unit-test.run', args={})],
        ),
        Phase(
            name='system-test',
            jobs=[Job(command='system-test.run', args={})],
        ),
        Phase(
            name='publish',
            plugins=['poetry'],
            jobs=[Job(command='poetry.publish', args={})],
        ),
        # Phase(
        #     name='deploy',
        #     plugins=['ansible'],
        #     jobs=[Job(command='system.run', args={})],
        # ),
    ]
)
namespace = Collection(
    formatting,
    install,
    static_analysis,
    build,
    unit_test,
    acceptance_test,
    integration_test,
    system_test,
    publish,
)
# pprint(asdict(project_config))
namespace.configure(asdict(project_config))
# namespace.load_collections()

__all__ = ['namespace']



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide PKI Task-Runner."""

import logging

from proman_workflows.collection import Collection

logging.getLogger(__name__).addHandler(logging.NullHandler())

# Assemble namespace for namespace
namespace = Collection()
namespace.load_collection(
    'gpg', driver_name='gpg', driver_namespace='proman.workflows.pki'
)
namespace.load_collection(
    'tls', driver_name='tls', driver_namespace='proman.workflows.pki'
)

__all__ = ['namespace']



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task(name='license')
def licensing(
    ctx,  # type: Context
    key=None,  # type: Optional[str]
    db=None,  # type: Optional[str]
    json=None,  # type: Optional[bool]
    bare=None,  # type: Optional[bool]
    cache=None,  # type: Optional[bool]
    file=None,  # type: Optional[str]
    proxy_host=None,  # type: Optional[str]
    proxy_port=None,  # type: Optional[int]
    proxy_protocol=None,  # type: Optional[str]
):  # type: (...) -> None
    """Configure license for safety."""
    args = []
    if key:
        args.append(f"--key={key}")
    if db:
        args.append(f"--db={db}")
    if json is not None:
        args.append('--json' if json else '--no-json')
    if bare is not None:
        args.append('--bare' if bare else '--not-bare')
    if cache is not None:
        args.append('--cache' if cache else '--no-cache')
    if file:
        args.append(f"--file={file}")
    if proxy_host:
        args.append(f"--proxy-host={proxy_host}")
    if proxy_port:
        args.append(f"--proxy-port={proxy_port}")
    if proxy_protocol:
        args.append(f"--proxy-protocol={proxy_protocol}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"safety license {' '.join(args)}")


@task
def review(
    ctx,  # type: Context
    full_report=None,  # type: Optional[bool]
    bare=None,  # type: Optional[bool]
    file=None,  # type: Optional[str]
):  # type: (...) -> None
    """Perform safety reiew of reports."""
    args = []
    if full_report is not None:
        args.append('--full-report' if full_report else '--short-report')
    if bare is not None:
        args.append('--bare' if bare else '--not-bare')
    if file:
        args.append(f"--file={file}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"safety review {' '.join(args)}")


@task
def check(
    ctx,  # type: Context
    key=None,  # type: Optional[str]
    db=None,  # type: Optional[str]
    json=None,  # type: Optional[bool]
    full_report=None,  # type: Optional[bool]
    bare=None,  # type: Optional[bool]
    cache=None,  # type: Optional[bool]
    stdin=None,  # type: Optional[bool]
    file=None,  # type: Optional[str]
    ignore=None,  # type: Optional[str]
    output=None,  # type: Optional[str]
    proxy_host=None,  # type: Optional[str]
    proxy_port=None,  # type: Optional[int]
    proxy_protocol=None,  # type: Optional[str]
):  # type: (...) -> None
    """Perform safety check of project dependencies."""
    args = []
    if key:
        args.append(f"--key={key}")
    if db:
        args.append(f"--db={db}")
    if json is not None:
        args.append('--json' if json else '--no-json')
    if full_report is not None:
        args.append('--full-report' if full_report else '--short-report')
    if bare is not None:
        args.append('--bare' if bare else '--not-bare')
    if cache is not None:
        args.append('--cache' if cache else '--no-cache')
    if stdin is not None:
        args.append('--stdin' if stdin else '--no-stdin')
    if file:
        args.append(f"--file={file}")
    if ignore:
        args.append(f"--ignore={ignore}")
    if output:
        args.append(f"--output={output}")
    if proxy_host:
        args.append(f"--proxy-host={proxy_host}")
    if proxy_port:
        args.append(f"--proxy-port={proxy_port}")
    if proxy_protocol:
        args.append(f"--proxy-protocol={proxy_protocol}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"safety check {' '.join(args)}")


namespace = Collection(check, licensing, review)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide package task-runner."""

# import importlib

from invoke import Collection

namespace = Collection()
# namespace.configure({})
# namespace.load_collections(
#     plugins=[
#         {
#             'name': 'exec',
#             'driver_name': 'briefcase',
#             'driver_namespace': 'proman.workflows.executable',
#         }
#     ]
# )



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def version(ctx):  # type: (Context) -> None
    """Get isort version number."""
    ctx.runt('isort --version-number')


@task
def run(
    ctx,  # type: Context
    atomic=True,  # type: bool
    check=False,  # type: bool
    diff=False,  # type: bool
    dedup_heading=False,  # type: bool
    jobs=None,  # type: Optional[int]
    ignore_whitespace=False,  # type: bool
    interactive=False,  # type: bool
    only_modified=False,  # type: bool
    overwrite_in_place=False,  # type: bool
    path=None,  # type: Optional[str]
    quiet=False,  # type: bool
    stdout=False,  # type: bool
    settings_path=None,  # type: Optional[str]
    profile=None,  # type: Optional[str]
    verbose=False,  # type: bool
):  # type: (...) -> None
    """Format project headers to PEP8 standards."""
    args = []
    if jobs:
        args.append(f"--jobs {jobs}")
    if check:
        args.append('--check-only')
    else:
        if atomic:
            args.append('--atomic')
        if dedup_heading:
            args.append('--dedup-headings')
        if overwrite_in_place:
            args.append('--overwrite-in-place')
    if diff:
        args.append('--diff')
    if ignore_whitespace:
        args.append('--ignore-whitespace')
    if settings_path:
        args.append(f"--settings-path {settings_path}")
    if profile:
        args.append(f"--profile {profile}")
    if quiet:
        args.append('--quiet')
    if stdout:
        args.append('--stdout')
    if verbose:
        args.append('--verbose')
    if only_modified:
        args.append('--only-modified')
    if interactive:
        args.append('--interactive')
    args.append(path if path else ctx.project_dir)
    ctx.run(f"isort {' '.join(args)}")


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide package task-runner."""

# import importlib
from typing import TYPE_CHECKING, Optional

from invoke import task

from ..collection import Collection

if TYPE_CHECKING:
    from invoke import Context


@task
def version(
    ctx,
    part=None,
    tag=False,
    commit=False,
    message=None,
):  # type: (Context, Optional[str], bool, bool, Optional[str]) -> None
    """Update project version and apply tags."""
    version = ctx.specfile['tool']['poetry']['version']
    if (
        'dev' in version
        or ('a' in version or 'alpha' in version)
        or ('b' in version or 'beta' in version)
        or 'rc' in version
    ):
        part = 'build'
    else:
        part = 'patch'

    args = [part]
    if commit:
        args.append('--commit')
    else:
        args.append('--dry-run')
        args.append('--allow-dirty')
        args.append('--verbose')
        print('Add "--commit" to actually bump the version.')
    if tag or message:
        args.append('--tag')
        if message:
            args.append(f"--tag-message '{message}'")
    ctx.run(f"bumpversion {' '.join(args)}")


namespace = Collection(
    configuration={
        'plugins': [
            {
                'name': 'package',
                'driver_name': 'poetry',
                'driver_namespace': 'proman.workflows.package',
            }
        ]
    }
)
namespace.add_task(version)



# -*- coding: utf-8 -*-
# copyright: (c) 2021 by Jesse Johnson.
# license: MPL 2.0, see LICENSE for more details.
"""Acceptance Tests task-runner using behave."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(
    ctx,  # type: Context
):  # type: (...) -> None
    """Perform unit tests."""
    print('behave stub')


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide flake8 Task-Runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(
    ctx,  # type: Context
    output_file=None,  # type: Optional[str]
    append_config=None,  # type: Optional[str]
    config=None,  # type: Optional[str]
    isolated=False,  # type: bool
    quiet=False,  # type: bool
    count=False,  # type: bool
    diff=False,  # type: bool
    exclude=None,  # type: Optional[str]
    extend_exclude=None,  # type: Optional[str]
    filename=None,  # type: Optional[str]
    stdin_display_name=None,  # type: Optional[str]
    kind=None,  # type: Optional[bool]
    hang_closing=None,  # type: Optional[bool]
    ignore=None,  # type: Optional[bool]
    extend_ignore=None,  # type: Optional[str]
    per_file_ignores=None,  # type: Optional[str]
    max_line_length=None,  # type: Optional[int]
    max_doc_length=None,  # type: Optional[int]
    indent_size=None,  # type: Optional[int]
    select=None,  # type: Optional[str]
    disable_noqa=False,  # type: bool
    show_source=None,  # type: Optional[bool]
    # no_show_source=None,  # type: Optional[bool]
    statistics=False,  # type: bool
    enable_extensions=None,  # type: Optional[str]
    exit_zero=False,  # type: bool
    jobs=None,  # type: Optional[str]
    tee=False,  # type: bool
    benchmark=False,  # type: bool
    bug_report=False,  # type: bool
    max_complexity=None,  # type: Optional[int]
    builtins=None,  # type: Optional[str]
    doctests=False,  # type: bool
    include_in_doctest=None,  # type: Optional[str]
    exclude_from_doctest=None,  # type: Optional[str]
):  # type: (...) -> None
    """Format project source code to PEP-8 standard."""
    args = []
    if output_file:
        args.append(f"--output-file {output_file}")
    if append_config:
        args.append(f"--append-config {append_config}")
    if config:
        args.append(f"--config {config}")
    if isolated:
        args.append('--isolated')
    if quiet:
        args.append('--quiet')
    if count:
        args.append('--count')
    if diff:
        args.append('--diff')
    if exclude:
        args.append(f"--exclude {exclude}")
    if extend_exclude:
        args.append(f"--extend-exclude {extend_exclude}")
    if filename:
        args.append(f"--filename {filename}")
    if stdin_display_name:
        args.append(f"--stdin-display-name {stdin_display_name}")
    if kind:
        args.append(f"--format {kind}")
    if hang_closing:
        args.append('--hang-closing')
    if ignore:
        args.append(f"--ignore {ignore}")
    if extend_ignore:
        args.append(f"--extend-ignore {extend_ignore}")
    if per_file_ignores:
        args.append(f"--per-file-ignores {per_file_ignores}")
    if max_line_length:
        args.append(f"--max-line-length {max_line_length}")
    if max_doc_length:
        args.append(f"--max-doc-length {max_doc_length}")
    if indent_size:
        args.append(f"--indent-size {indent_size}")
    if select:
        args.append(f"--select {select}")
    if disable_noqa:
        args.append('--disable-noqa')
    if show_source is not None:
        args.append('--show-source' if show_source else '--no-show-source')
    if statistics:
        args.append('--statistics')
    if enable_extensions:
        args.append(f"--enable-extensions {enable_extensions}")
    if exit_zero:
        args.append('--exit-zero')
    if jobs:
        args.append(f"--jobs {jobs}")
    if tee:
        args.append('--tee')
    if benchmark:
        args.append('--benchmark')
    if bug_report:
        args.append('--bug-report')
    if max_complexity:
        args.append(f"--max-complexity {max_complexity}")
    if builtins:
        args.append(f"--builtins {builtins}")
    if doctests:
        args.append('--doctests')
    if include_in_doctest:
        args.append(f"--include-in-doctest {include_in_doctest}")
    if exclude_from_doctest:
        args.append(f"--exclude-from-doctest {exclude_from_doctest}")
    with ctx.cd(ctx.project_dir):
        ctx.run(f"flake8 {' '.join(args)}")


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

import logging

from proman_workflows.collection import Collection

logging.getLogger(__name__).addHandler(logging.NullHandler())

# Assemble collections for namespace
namespace = Collection(
    configuration={
        'plugins': [
            {
                'name': 'unit-test',
                'driver_name': 'pytest',
                'driver_namespace': 'proman.workflows.qa',
            },
            {
                'name': 'acceptance-test',
                'driver_name': 'behave',
                'driver_namespace': 'proman.workflows.qa',
            },
        ]
    }
)

__all__ = ['namespace']



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide documentation task-runner."""

from typing import TYPE_CHECKING, Optional

from invoke import task

from ..collection import Collection

if TYPE_CHECKING:
    from invoke import Context


@task(iterable='ignore_decorators')
def style(
    ctx,  # type: Context
    explain=False,  # type: bool
    source=False,  # type: bool
    count=False,  # type: bool
    configpath=None,  # type: Optional[str]
    match=None,  # type: Optional[str]
    match_dir=None,  # type: Optional[str]
    ignore_decorators=None,  # type: Optional[str]
    debug=False,  # type: bool
    verbose=False,  # type: bool
    version=False,  # type: bool
):  # type: (...) -> None
    """Check code for documentation errors."""
    args = []
    if configpath:
        args.append(f"--config={configpath}")
    if explain:
        args.append('--explain')
    if source:
        args.append('--source')
    if count:
        args.append('--count')
    if match:
        args.append(f"--match={match}")
    if match_dir:
        args.append(f"--match-dir={match_dir}")
    if ignore_decorators:
        args.append(f"--ignore-decorators={ignore_decorators}")
    if debug:
        args.append('--debug')
    if verbose:
        args.append('--verbose')
    if version:
        args.append('--version')
    ctx.run(f"pydocstyle {' '.join(args)}")


@task(incrementable=['verbose'])
def coverage(
    ctx,  # type: Context
    exclude=None,  # type: Optional[str]
    skipmagic=False,  # type: bool
    skipfiledoc=False,  # type: bool
    skipinit=False,  # type: bool
    skipclassdef=False,  # type: bool
    skip_private=False,  # type: bool
    followlinks=False,  # type: bool
    docstr_ignore_file=None,  # type: Optional[str]
    failunder=90,  # type: int
    badge=None,  # type: Optional[str]
    percentage_only=False,  # type: bool
    verbose=None,  # type: Optional[int]
    path='.',  # type: str
):  # type: (...) -> None
    """Ensure all code is documented."""
    args = [ctx.project_dir]
    if exclude:
        args.append(f"--exclude '{exclude}'")
    if skipmagic:
        args.append('--skipmagic')
    if skipfiledoc:
        args.append('--skipfiledoc')
    if skipinit:
        args.append('--skipinit')
    if skipclassdef:
        args.append('--skipclassdef')
    if skip_private:
        args.append('--skip-private')
    if followlinks:
        args.append('--followlinks')
    if docstr_ignore_file:
        args.append(f"--docstr-ignore-file {docstr_ignore_file}")
    if failunder:
        args.append(f"--failunder {failunder}")
    if badge:
        args.append(f"--badge {badge}")
    if percentage_only:
        args.append('--percentage-only')
    if verbose:
        args.append(f"--verbose {verbose}")
    ctx.run(f"docstr-coverage {' '.join(args)}")


namespace = Collection(style, coverage)
# namespace.configure({})
namespace.load_collections(
    plugins=[
        {
            'name': 'site',
            'driver_name': 'mkdocs',
            'driver_namespace': 'proman.workflows.docs',
        }
    ]
)

__all__ = ['style', 'coverage', 'namespace']



"""Provide templating capability."""

# import os
from typing import TYPE_CHECKING, Any, Dict

from invoke import task
from jinja2 import Environment, FileSystemLoader

if TYPE_CHECKING:
    from invoke import Context


@task
def render(
    ctx, data, template_name
):  # type: (Context, Dict[str, Any], str) -> str
    """Render template."""
    loader = FileSystemLoader(ctx.templates_dir)
    env = Environment(loader=loader, autoescape=True)
    template = env.get_template(f"{template_name}.j2")
    content = template.render(**data)
    return content



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(ctx, path='.'):  # type: (Context, str) -> None
    """Check project source types."""
    with ctx.cd(ctx.project_dir):
        ctx.run(f"mypy {path}")


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

import logging

from ..collection import Collection
from . import security

logging.getLogger(__name__).addHandler(logging.NullHandler())

# Assemble collections for namespace
namespace = Collection(
    configuration={
        'plugins': [
            {
                'name': 'lint',
                'driver_name': 'flake8',
                'driver_namespace': 'proman.workflows.sca',
            },
            {
                'name': 'type-checking',
                'driver_name': 'mypy',
                'driver_namespace': 'proman.workflows.sca',
            },
            # bandit = "proman_workflows.sca.security.bandit:namespace"
            # safety = "proman_workflows.sca.security.safety:namespace"
        ]
    }
)
namespace.add_collection(security.namespace, name='sec')

__all__ = ['namespace']



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide documentation task-runner."""

from ..collection import Collection

namespace = Collection(
    configuration={
        'plugins': [
            # {
            #     'name': 'docker',
            #     'driver_name': 'docker_compose',
            #     'driver_namespace': 'proman.workflows.container.compose',
            # },
            {
                'name': 'podman',
                'driver_name': 'podman_compose',
                'driver_namespace': 'proman.workflows.container.compose',
            },
        ]
    }
)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide utilities task-runner."""

# import importlib
from typing import TYPE_CHECKING, List, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context, Result


@task
def find(
    ctx,  # type: Context
    name,  # type: str
    path='.',  # type: str
    mindepth=None,  # type: Optional[int]
    maxdepth=None,  # type: Optional[int]
):  # type: (...) -> List[Result]
    """Clean project dependencies and build."""
    args = []
    if mindepth:
        args.append(f"-mindepth {mindepth}")
    if maxdepth:
        args.append(f"-maxdepth {maxdepth}")
    result = ctx.run(f"find {path} {' '.join(args)}")
    return result


@task(iterable=['path'])
def clean(
    ctx,  # type: Context
    path=None,  # type: Optional[str]
    mindepth=None,  # type: Optional[int]
    maxdepth=None,  # type: Optional[int]
):  # type: (...) -> None
    """Clean project dependencies and build."""
    args = []
    if not path:
        paths = [
            '__pycache__',
            '.mypy_cache',
            'dist',
            '*.pyc',
        ]
    if mindepth:
        args.append(f"-mindepth {mindepth}")
    if maxdepth:
        args.append(f"-maxdepth {maxdepth}")
    for path in paths:
        ctx.run(
            "find . %s -exec rm -rf {} +"
            % (' '.join([f"-name {path}"] + args))
        )


@task(iterable=['runstates'])
def pkill(
    ctx,  # type: Context
    pattern,  # type: str
    signal=None,  # type: int
    echo=False,  # type: bool
    count=False,  # type: bool
    full=False,  # type: bool
    pgroup=None,  # type: Optional[str]
    group=None,  # type: Optional[str]
    ignore_case=False,  # type: bool
    newest=False,  # type: bool
    oldest=False,  # type: bool
    parent=None,  # type: Optional[str]
    session=None,  # type: Optional[str]
    terminal=None,  # type: Optional[str]
    euid=None,  # type: Optional[str]
    uid=None,  # type: Optional[str]
    exact=False,  # type: bool
    pidfile=None,  # type: Optional[str]
    logpidfile=False,  # type: bool
    runstates=[],  # type: List[str]
    namespace=None,  # type: Optional[str]
    nslist=None,  # type: Optional[str]
):  # type: (...) -> None
    """Stop docsite."""
    args = []
    if signal:
        args.append(f"--signal {signal}")
    if echo:
        args.append('--echo')
    if count:
        args.append('--count')
    if full:
        args.append('--full')
    if pgroup:
        args.append(f"--pgroup {pgroup}")
    if group:
        args.append(f"--group {group}")
    if ignore_case:
        args.append('--ignore-case')
    if newest:
        args.append('--newest')
    if oldest:
        args.append('--oldest')
    if parent:
        args.append(f"--parent {parent}")
    if session:
        args.append(f"--session {session}")
    if terminal:
        args.append(f"--terminal {terminal}")
    if euid:
        args.append(f"--euid {euid}")
    if uid:
        args.append(f"--uid {uid}")
    if exact:
        args.append('--exact')
    if pidfile:
        args.append(f"--pidfile {pidfile}")
    if logpidfile:
        args.append('--logpidfile')
    if runstates != []:
        args.append(f"--runstates {','.join(runstates)}")
    if namespace:
        args.append(f"--ns {namespace}")
    if nslist:
        # Available namespaces: ipc, mnt, net, pid, user, uts
        args.append(f"--nslist {nslist}")
    args.append(pattern)
    ctx.run(f"pkill {' '.join(args)}")


namespace = Collection(clean)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Provide black integration task-runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def run(
    ctx,  # type: Context
    path='**/*.py',  # type: str
    code=None,  # type: Optional[str]
    line_length=79,  # type: int
    target_version=None,  # type: Optional[str]
    pyi=False,  # type: bool
    skip_string_normalization=True,  # type: bool
    skip_magic_trailing_comma=False,  # type: bool
    check=False,  # type: bool
    diff=False,  # type: bool
    color=None,  # type: Optional[bool]
    execution=None,  # type: Optional[bool]
    required_version=None,  # type: Optional[str]
    include=None,  # type: Optional[str]
    exclude=None,  # type: Optional[str]
    extend_exclude=None,  # type: Optional[str]
    force_exclude=None,  # type: Optional[str]
    quiet=None,  # type: Optional[bool]
    verbose=None,  # type: Optional[bool]
    version=None,  # type: Optional[bool]
    config=None,  # type: Optional[str]
):  # type: (...) -> None
    """Lint or format using black."""
    args = []
    if code:
        args.append(f"--code {code}")
    if line_length:
        args.append(f"--line-length {line_length}")
    if target_version:
        args.append(f"--target-version {target_version}")
    if pyi:
        args.append('--pyi')
    if skip_string_normalization:
        args.append('--skip-string-normalization')
    if skip_magic_trailing_comma:
        args.append('--skip-magic-trailing-comma')
    if check:
        args.append('--check')
    if diff:
        args.append('--diff')
    if color is not None:
        args.append('--color' if color else '--no-color')
    if execution == 'fast':
        args.append('--fast')
    if execution == 'safe':
        args.append('--safe')
    if required_version:
        args.append(f"--required-version {required_version}")
    if include:
        args.append(f"--include {include}")
    if exclude:
        args.append(f"--exclude {exclude}")
    if extend_exclude:
        args.append(f"--extend-exclude {extend_exclude}")
    if force_exclude:
        args.append(f"--force-exclude {force_exclude}")
    if quiet:
        args.append('--quiet')
    if verbose:
        args.append('--verbose')
    if version:
        args.append('--version')
    if config:
        args.append(f"--config {config}")
    args.append(path)
    ctx.run(f"black {' '.join(args)}")


namespace = Collection(run)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Validation Task-Runner."""

# import platform
from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

from .. import filesystem

# from .package_manager import github

if TYPE_CHECKING:
    from invoke import Context

# mkcert = f"mkcert-{config.mkcert_version}-{config.system_type}-amd64"


@task
def setup(ctx):  # type: (Context) -> None
    """Install GitHub release."""
    # github.install(
    #     ctx,
    #     repo='FiloSottile/mkcert',
    #     version=config.mkcert_version,
    #     filename=mkcert,
    #     new_filename='mkcert',
    #     file_pattern=f"*{platform.system().lower()}-amd64*"
    # )
    ctx.run('mkcert -install')


@task(pre=[setup], iterable=['name'])
def generate(
    ctx,  # type: Context
    name,  # type: str
    key=None,  # type: Optional[str]
    cert=None,  # type: Optional[str]
    p12=None,  # type: Optional[str]
    client=None,  # type: Optional[str]
    csr=None,  # type: Optional[str]
    ecdsa=False,  # type: bool
    pkcs12=False,  # type: bool
    path=None,  # type: Optional[str]
):  # type: (...) -> None
    """Generate certificate."""
    args = []
    if key:
        args.append(f"-key-file={key}")
    if cert:
        args.append(f"-cert-file={cert}")
    if p12:
        args.append(f"-client={client}")
    if ecdsa:
        args.append(f"-ecdsa={ecdsa}")
    if pkcs12:
        args.append(f"-pkcs12={pkcs12}")
    if csr:
        args.append(f"-csr={csr}")
    cmd = f"mkcert {' '.join(args)} {' '.join(name)}"
    if path:
        filesystem.mkdir(ctx, path)
        with ctx.cd(path):
            ctx.run(cmd)
    else:
        ctx.run(cmd)


@task
def clean(ctx, path=None):  # type: (Context, Optional[str]) -> None
    """Cleanup certificates."""
    if path:
        filesystem.rm(ctx, path)
    ctx.run('mkcert -uninstall')


namespace = Collection(clean, generate, setup)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['proman_workflows',
 'proman_workflows.container',
 'proman_workflows.docs',
 'proman_workflows.executable',
 'proman_workflows.formatter',
 'proman_workflows.git',
 'proman_workflows.package',
 'proman_workflows.pki',
 'proman_workflows.qa',
 'proman_workflows.sca',
 'proman_workflows.sca.security',
 'proman_workflows.system',
 'proman_workflows.templates',
 'proman_workflows.web']

package_data = \
{'': ['*'],
 'proman_workflows': ['_vendor/gitignore/*',
                      '_vendor/gitignore/.github/*',
                      '_vendor/gitignore/Global/*',
                      '_vendor/gitignore/community/*',
                      '_vendor/gitignore/community/AWS/*',
                      '_vendor/gitignore/community/DotNet/*',
                      '_vendor/gitignore/community/Elixir/*',
                      '_vendor/gitignore/community/GNOME/*',
                      '_vendor/gitignore/community/Golang/*',
                      '_vendor/gitignore/community/Java/*',
                      '_vendor/gitignore/community/JavaScript/*',
                      '_vendor/gitignore/community/Linux/*',
                      '_vendor/gitignore/community/PHP/*',
                      '_vendor/gitignore/community/Python/*',
                      '_vendor/gitignore/community/embedded/*']}

install_requires = \
['PyInputPlus>=0.2.12,<0.3.0',
 'briefcase>=0.3.5,<0.4.0',
 'compendium>=0.1.1-alpha.0,<0.2.0',
 'editables>=0.2,<0.3',
 'invoke>=1.5.0,<2.0.0',
 'jinja2>=2.11.2,<3.0.0',
 'keyring>=23.2.1,<24.0.0',
 'proman-common>=0.1.1-alpha.1,<0.2.0',
 'pygit2>=1.7,<2.0',
 'python-gnupg>=0.4.7,<0.5.0',
 'stevedore>=3.4.0,<4.0.0']

extras_require = \
{':python_version >= "3.7" and python_version < "3.8"': ['cached-property>=1.5.2,<2.0.0'],
 ':sys_platform == "linux"': ['SecretStorage>=3.3.1,<4.0.0']}

entry_points = \
{'console_scripts': ['workflow = proman_workflows:workflow.run',
                     'workflow-setup = proman_workflows.setup:setup.run',
                     'workflow-tools = proman_workflows:workflow_tools.run'],
 'proman.workflows.container.compose': ['docker_compose = '
                                        'proman_workflows.container.docker:namespace',
                                        'podman_compose = '
                                        'proman_workflows.container.podman:namespace'],
 'proman.workflows.docs': ['mkdocs = proman_workflows.docs.mkdocs:namespace'],
 'proman.workflows.executable': ['briefcase = '
                                 'proman_workflows.executable.briefcase:namespace'],
 'proman.workflows.formatter': ['autopep8 = '
                                'proman_workflows.formatter.autopep8:namespace',
                                'black = '
                                'proman_workflows.formatter.black:namespace',
                                'isort = '
                                'proman_workflows.formatter.isort:namespace'],
 'proman.workflows.mock': ['original = mock_workflow.mock.original:namespace',
                           'update = mock_workflow.mock.update:namespace'],
 'proman.workflows.package': ['flit = proman_workflows.package.flit:namespace',
                              'poetry = '
                              'proman_workflows.package.poetry:namespace',
                              'setuptools = '
                              'proman_workflows.package.setuptools:namespace',
                              'twine = '
                              'proman_workflows.package.twine:namespace'],
 'proman.workflows.pki': ['gpg = proman_workflows.pki.gpg:namespace',
                          'tls = proman_workflows.pki.tls:namespace'],
 'proman.workflows.qa': ['behave = proman_workflows.qa.behave:namespace',
                         'pytest = proman_workflows.qa.pytest:namespace'],
 'proman.workflows.sca': ['bandit = '
                          'proman_workflows.sca.security.bandit:namespace',
                          'flake8 = proman_workflows.sca.flake8:namespace',
                          'mypy = proman_workflows.sca.mypy:namespace',
                          'safety = '
                          'proman_workflows.sca.security.safety:namespace'],
 'proman.workflows.system': ['test-infra = '
                             'proman_workflows.system.test_infra:namespace'],
 'proman.workflows.vcs': ['git = proman_workflows.git:namespace']}

setup_kwargs = {
    'name': 'proman-workflows',
    'version': '0.1.0a5',
    'description': 'Task-runner to automate projects.',
    'long_description': '# proman-workflows\n\n[![License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://spdx.org/licenses/MPL-2.0)\n[![Build Status](https://travis-ci.org/python-proman/proman-workflows.svg?branch=master)](https://travis-ci.org/python-proman/proman-workflows)\n[![codecov](https://codecov.io/gh/python-proman/proman-workflows/branch/master/graph/badge.svg)](https://codecov.io/gh/python-proman/proman-workflows)\n\n## Overview\n\nThis project is a task runner to help automate common development tasks for\nprojects using Python.\n\nThe goal of this effort is help implement DevSecOps practices consistly with\nthe Software Development Lifecycle (SDLC) without burdening developers.\n\nThe objectives for achieving this goal are:\n- Enforce use of commit signing\n- Introduce QA tools early in development\n- SAST, SCA, and DAST integration\n- Encapulate processes for seamless integration with CI/CD systems\n- Make TUF compliant packages for PyPI\n\n## Install\n\nThe project can be installed using the following command:\n\n```\npip install proman-workflows\n```\n\nThe above will only install the workflows but not all dependencies. The\nadditional dependencies can be install with:\n\n```\npip install proman-workflow[all]\n```\n\n### Usage\n\nCurrently, there are three command line utilities included with this install.\nThis is due to the primary CLI tool being under heavy development.\n\nThe `workflow-tools` command provides direct access to each of integrations\nprovided by the task runner. It can either be used directory or extended as\na library for additional workflows.\n\nThe `workflow-setup` command\n\nThe `workflow` command is the intended CLI for the task runner but is still\nunder development. It will allow control of integrated tools through abstracted\nphases accessible to a developer. The functionality is still imited at this time.\n\n### Setup\n\nSetup a signing key for development:\n\n```\nworkflow-tools setup\n```\n\n## FAQ\n\nQ: Why should developers use this?\nA: Coodinating procedures and setup for multiple team members and projects is difficult and error\nprone. Task runners are purpose built to solve this problem.\n\nQ: Why not include this using project templates?\nA: Since this is distributed as a library updates and changes can be much more easilly distributed.\n\nQ: Why not use Invocations\nA: While this project is inspired by Invocations, it does not support a pluggable architecture.\n\n## Refereces\n\n- https://theupdateframework.io/\n- https://nvlpubs.nist.gov/nistpubs/CSWP/NIST.CSWP.04232020.pdf\n',
    'author': 'Jesse P. Johnson',
    'author_email': 'jpj6652@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/python-protools/proman-workflows',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Provide templating for git hooks."""

import os
import shutil
from typing import TYPE_CHECKING, List

from invoke import Collection, task

# from proman_workflows import git, filesystem, templates
from . import config

if TYPE_CHECKING:
    from invoke import Context


@task(name='list')
def index(ctx):  # type: (Context) -> List[str]
    """List git hooks."""
    hooks = [
        x
        for x in os.listdir(os.path.join(ctx.repo_dir, 'hooks'))
        if not x.endswith('sample')
    ]
    for hook in hooks:
        print(hook)
    return hooks


@task
def setup(
    ctx, name='pre-commit', update=False
):  # type: (Context, str, bool) -> None
    """Create git hook."""
    path = os.path.join(ctx.hooks_dir, name)
    if update or not os.path.exists(path):
        data = {
            'executable': shutil.which('project'),
            'hook': name,
        }
        config.dump(
            ctx,
            data,
            template_name='githooks',
            dest=path,
            executable=True,
            update=update,
        )


@task
def remove(ctx, name='pre-commit'):  # type: (Context, str) -> None
    """Remove git hook."""
    path = os.path.join(ctx.hooks_dir, name)
    if os.path.exists(path):
        os.remove(path)


@task
def execute(ctx, name):  # type: (Context, str) -> None
    """Execute task for githook."""
    # TODO setup tasks config for executor
    print(f"{name} hook executed")


namespace = Collection(execute, index, remove, setup)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide CLI management."""

from proman_workflows import workflow

if __name__ == '__main__':
    workflow.run()



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Validation Task-Runner."""

import os
import shutil
import stat
from typing import TYPE_CHECKING, Optional

from invoke import task

if TYPE_CHECKING:
    from invoke import Context


# async def rwalk(
#     path: str, stop: Optional[str] = None
# ) -> AsyncGenerator[str, str]:
#     """Walk the reverse path of a directory."""
#     current_path = os.path.abspath(path)
#     for _ in path.split(os.sep):
#         yield current_path
#         current_path = os.path.abspath(os.path.join(current_path, '..'))


@task
def write(ctx, content, dest, executable=False, update=False):
    # type: (Context, str, str, bool, bool) -> None
    """Create git hook."""
    if update or not os.path.exists(dest):
        with open(dest, 'w+') as f:
            f.write(content)

        if executable:
            st = os.stat(dest)
            os.chmod(dest, st.st_mode | stat.S_IEXEC)


@task
def mkdir(ctx, path):  # type: (Context, str) -> None
    """Make directory path."""
    try:
        os.makedirs(path, exist_ok=True)
    except OSError as err:
        print(f"unable to download github release due to: {err}")


@task
def rm(ctx, path):  # type: (Context, str) -> None
    """Remove directory path."""
    if os.path.exists(path):
        if os.path.isfile(path):
            os.remove(path)
        if os.path.isdir(path):
            try:
                shutil.rmtree(path)
            except OSError as err:
                print(f"unable to delete direcotry path due to: {err}")
    else:
        print(f"path does not exists: {path}")


@task(iterable=['path'])
def clean(
    ctx,  # type: Context
    path=None,  # type: Optional[str]
    mindepth=None,  # type: Optional[int]
    maxdepth=None,  # type: Optional[int]
):  # type: (...) -> None
    """Clean project dependencies and build."""
    args = []
    if not path:
        paths = [
            '__pycache__',
            '.mypy_cache',
            'dist',
            '*.pyc',
        ]
    if mindepth:
        args.append(f"-mindepth {mindepth}")
    if maxdepth:
        args.append(f"-maxdepth {maxdepth}")
    for path in paths:
        ctx.run(
            "find . %s -exec rm -rf {} +"
            % (' '.join([f"-name {path}"] + args))
        )



"""Provide git config management."""

import os
from typing import TYPE_CHECKING, Any, Dict, Optional

from invoke import Collection, task
from pygit2 import Config, Repository

from .. import filesystem, templates

if TYPE_CHECKING:
    from invoke import Context


def to_dict(config: Config) -> Dict[str, Any]:
    """Convert gitconfig to dictionary."""
    sections: Dict[str, Any] = {}
    for entry in config:
        s = sections
        if hasattr(entry, 'name'):
            items = entry.name.split('.')
            for index, part in enumerate(items):
                if index < (len(items) - 1):
                    s = s.setdefault(part, {})
                else:
                    s[part] = entry.value
    return {'sections': sections}


@task
def repo(ctx):  # type: (Context) -> Repository
    """Set git as SCM."""
    # XXX: can set context using set
    return Repository(ctx.repo_dir)


@task
def dump(
    ctx, data, template_name, dest, executable=False, update=False
):  # type: (Context, Dict[str, Any], str, str, bool, bool) -> None
    """Create git hook."""
    if update or not os.path.exists(dest):
        content = templates.render(ctx, data=data, template_name=template_name)
        filesystem.write(
            ctx,
            content=content,
            dest=dest,
            executable=executable,
            update=update,
        )
    else:
        print('Config already exists.')


@task
def load(
    ctx, filepath=None, scope=None
):  # type: (Context, Optional[str], Optional[str]) -> Dict[str, Any]
    """Get git config as dictionary."""
    vcs = repo(ctx)
    if scope == 'system':
        config = vcs.config.get_system_config()
    elif scope == 'global':
        config = vcs.config.get_global_config()
    elif scope == 'xdg':
        config = vcs.config.get_xdg_config()
    else:
        config = Config(filepath or os.path.join(ctx.repo_dir, 'config'))
    return to_dict(config)


@task
def setup(
    ctx, filepath=None, scope=None, update=False
):  # type: (Context, Optional[str], Optional[str], bool) -> None
    """Set VCS configuration."""
    data = load(ctx, filepath=filepath, scope=scope)

    # if 'user.name' not in config:
    #     username = pyip.inputStr(
    #         prompt='GIT: Enter full name: ',
    #         limit=255,
    #     )
    #     config.set_multivar('user.name', regex='^.*$', value=username)
    #     print('user.name', config.get_multivar('user.name'))
    # if 'user.email' not in config:
    #     email = pyip.inputEmail(
    #         prompt='GIT: Enter email: ',
    #         limit=255,
    #     )
    #     config.set_multivar('user.email', regex='^.*$', value=email)
    #     print('user.email', config.get_multivar('user.email'))

    dump(
        ctx,
        data,
        template_name='gitconfig',
        dest=os.path.join(ctx.repo_dir, 'check'),
        update=update,
    )


config_tasks = Collection(dump, load)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Build Task-Runner."""

from typing import TYPE_CHECKING, Any, Dict, List, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context

# TODO: switch to podman module


@task(iterable=['files'])
def start(
    ctx,  # type: Context
    files,  # type: List[str]
    env={},  # type: Dict[str, Any]
    build=False,  # type: bool
    recreate=False,  # type: bool
    dev=False,  # type: bool
):  # type: (...) -> None
    """Start all compose containers."""
    args = []
    if build:
        args.append('--build')
    if recreate:
        args.append('--force-recreate')

    if dev:
        files.append('docker-compose.yml')
        files.append('docker-compose.override.yml')

    if files != []:
        files = [f"--file={f}" for f in files]

    with ctx.cd(ctx.container_build_dir):
        ctx.run(
            "podman-compose {f} up -d {a}".format(
                f='' if files == [] else ' '.join(files), a=' '.join(args)
            ),
            env=env,
        )


@task
def stop(
    ctx,  # type: Context
    remove_images=None,  # type: Optional[str]
    remove_volumes=True,  # type: bool
    remove_orphans=True,  # type: bool
):  # type: (...) -> None
    """Stop all compose containers."""
    args = []
    if remove_images:
        args.append(f"--rmi={remove_images}")
    if remove_volumes:
        args.append('--volumes')
    if remove_orphans:
        args.append('--remove-orphans')
    with ctx.cd(ctx.container_build_dir):
        ctx.run(f"podman-compose down {' '.join(args)}")


namespace = Collection(start, stop)



"""Provide signature capability using GnuPG."""

import os
from dataclasses import asdict
from typing import TYPE_CHECKING, Any, Dict, List, Optional

from gnupg import GPG
from invoke import Collection, task

from proman_workflows.config import GPGConfig

if TYPE_CHECKING:
    from gnupg import GenKey, ListKeys
    from invoke import Context

ELYPTICAL_KEY_TYPES = [
    'ECDH',
    'ECDSA',
    'EDDSA',
]
KEY_TYPES = [
    'DSA',
    'ELG',
    'RSA',
] + ELYPTICAL_KEY_TYPES

# TODO: need better way of mapping curves
# ELYPTICAL_CURVES = [
#     'curve25519',  # cv25519, or ed25519
#     'nistp256',  # nothing up my sleeve?
#     'nistp384',  # nothing up my sleeve?
#     'nistp521',  # nothing up my sleeve?
#     'secp256k1',
#     # 'brainpoolP256r1',  # verifiable?
#     # 'brainpoolP384r1',  # verifiable?
#     # 'brainpoolP512r1',  # verifiable?
# ]


@task
def agent(
    ctx,  # type: Context
    daemon=False,  # type: bool
    supervised=False,  # type: bool
    verbose=False,  # type: bool
    quiet=False,  # type: bool
    sh=False,  # type: bool
    csh=False,  # type: bool
    options=None,  # type: Optional[str]
    dettach=True,  # type: bool
    log_file=None,  # type: Optional[str]
    pinentry_program=None,  # type: Optional[str]
    allow_loopback_pinentry=True,  # type: bool
    allow_emacs_pinentry=True,  # type: bool
    scdaemon_program=None,  # type: Optional[str]
    disable_scdaemon=True,  # type: bool
    extra_socket=None,  # type: Optional[int]
    keep_tty=True,  # type: bool
    keep_display=False,  # type: bool
    default_cache_ttl=None,  # type: Optional[int]
    ignore_cache_for_signing=False,  # type: bool
    allow_external_cache=True,  # type: bool
    allow_mark_trusted=True,  # type: bool
    allow_preset_passphrase=False,  # type: bool
    enable_ssh_support=False,  # type: bool
    ssh_fingerprint_digest=None,  # type: Optional[str]
):  # type: (...) -> None
    """Start gpg agent."""
    args = []
    if daemon:
        args.append('--daemon')
    if supervised:
        args.append('--supervised')
    if verbose:
        args.append('--verbose')
    if quiet:
        args.append('--quiet')

    # shell
    if sh:
        args.append('--sh')
    if csh:
        args.append('--csh')

    if keep_tty:
        args.append('--keep-tty')
    if keep_display:
        args.append('--keep-display')

    if not dettach:
        args.append('--no-detach')
    if log_file:
        args.append('--log-file')

    # setup pin entry UI application
    if pinentry_program:
        args.append(f"--pinentry-program {pinentry_program}")
        if not allow_loopback_pinentry:
            args.append('--no-allow-loopback-pinentry')
        if allow_emacs_pinentry:
            args.append('--allow-emacs-pinentry')

    # setup smartcard integration
    if disable_scdaemon:
        args.append('--disable-scdaemon')
    elif scdaemon_program:
        args.append(f"--scdaemon-program {scdaemon_program}")

    if extra_socket:
        args.append(f"--extra-socket {extra_socket}")

    # cache
    if default_cache_ttl:
        args.append(f"--default-cache-ttl {default_cache_ttl}")
    if ignore_cache_for_signing:
        args.append('--ignore-cache-for-signing')
    if not allow_external_cache:
        args.append('--no-allow-external-cache')

    if not allow_mark_trusted:
        args.append('--no-allow-mark-trusted')
    if allow_preset_passphrase:
        args.append('--allow-preset-passphrase')
    if enable_ssh_support:
        args.append('--enable-ssh-support')
        if ssh_fingerprint_digest:
            args.append(f"--ssh-fingerprint-digest {ssh_fingerprint_digest}")

    # gpg options
    if options:
        args.append(f"--options '{','.join(options)}'")
    ctx.run(f"gpgconf --launch gpg-agent {' '.join(args)}")


@task
def gpg(
    ctx,  # type: Context
    gnupghome=os.path.join(os.path.expanduser('~'), '.gnupg'),  # type: str
    keyring=None,  # type: Optional[str]
    secret_keyring=None,  # type: Optional[str]
):  # type: (...) -> None
    """Get gpg instance."""
    if '__gpg' not in ctx:
        ctx.__gpg = GPG(
            gnupghome=gnupghome,
            keyring=keyring,
            secret_keyring=secret_keyring
        )


@task(pre=[gpg])
def list_keys(
    ctx, secret=False, keys=None, sigs=False
):  # type: (Context, bool, Optional[str], bool) -> ListKeys
    """List GPG keys."""
    keys = ctx.__gpg.list_keys(secret=secret, keys=keys, sigs=sigs)
    print([k['keyid'] for k in keys])  # type: ignore
    return keys


@task(pre=[gpg])
def get_key(
    ctx, query, secret=False, sigs=False
):  # type: (Context, str, bool, bool) -> List[GenKey]
    """Get GPG key."""
    keys = []
    for key in list_keys(ctx, secret=secret, sigs=sigs):
        if '=' in query:
            k, v = query.split('=')
            if key[k] == v:
                keys.append(key)
        elif query in key.values():
            keys.append(key)
    return keys


@task(pre=[gpg])
def export_key(
    ctx, key, keypath, secret=False
):  # type: (Context, str, str, bool) -> None
    """Export an armored GPG key."""
    ascii_armored_keys = ctx.__gpg.export_keys(key, secret)
    with open(f"{key}.asc", 'w') as f:
        f.write(ascii_armored_keys)


@task(pre=[gpg])
def delete_key(
    ctx,
    query,
    passphrase,
    # secret=True,
    sigs=False,
    expect_passphrase=True,
):  # type: (Context, str, str, bool, bool) -> None
    """Delete GPG key."""
    for secret in (True, False):
        keys = get_key(ctx, query, secret, sigs)
        for key in keys:
            ctx.__gpg.delete_keys(
                [x['fingerprint'] for x in keys],
                secret=secret,
                passphrase=passphrase,
                expect_passphrase=expect_passphrase,
            )


@task(pre=[gpg], iterable=['key_usage', 'subkey_usage'])
def gen_key(
    ctx,  # type: Context
    name,  # type: str
    email,  # type: str
    password,  # type: str
    comment='generated by proman',  # type: Optional[str]
    key_type='EDDSA',  # type: str
    key_curve='ed25519',  # type: str
    key_length=None,  # type: Optional[int]
    key_usage=None,  # type: Optional[List[str]]
    subkey_type='EDDSA',  # type: str
    subkey_curve='ed25519',  # type: str
    subkey_length=None,  # type: Optional[int]
    subkey_usage=None,  # type: Optional[List[str]]
    expire_date='1y',  # type: str
):  # type: (...) -> GenKey
    """Generate GPG key."""
    # TODO: default to subkey if primary key exists
    usage = ['auth', 'cert', 'encrypt', 'sign']
    settings: Dict[str, Any] = {}

    # separate_keyring
    # save_batchfile
    # testing
    # key_grip
    # creation_date
    # preferences
    # revoker
    # keyserver
    # handle

    settings['key_type'] = key_type
    if key_type not in ELYPTICAL_KEY_TYPES:
        settings['key_length'] = key_length
    elif key_type in ELYPTICAL_KEY_TYPES:
        if key_curve:
            # and key_curve in KEY_CURVES:
            settings['key_curve'] = key_curve
    if key_usage and all(item in key_usage for item in usage):
        settings['key_usage'] = ','.join(key_usage)

    settings['subkey_type'] = subkey_type
    if subkey_type not in ELYPTICAL_KEY_TYPES:
        settings['subkey_length'] = subkey_length
    elif subkey_type in ELYPTICAL_KEY_TYPES:
        if subkey_curve:
            # and subkey_curve in KEY_CURVES:
            settings['subkey_curve'] = subkey_curve
    if subkey_usage and all(item in subkey_usage for item in usage):
        settings['subkey_usage'] = ','.join(subkey_usage)

    settings['expire_date'] = expire_date

    input_data = ctx.__gpg.gen_key_input(
        name_real=name,
        name_email=email,
        name_comment=comment,
        passphrase=password,
        **settings,
    )
    print('input', input_data)
    key = ctx.__gpg.gen_key(input_data)
    # print('key', key)
    return key


namespace = Collection(
    list_keys,
    get_key,
    delete_key,
    export_key,
    gen_key,
)
namespace.configure(
    {
        'settings': asdict(GPGConfig()),
        'signing': {
            'developers': {
                'required': True,
            },
            'packaging': {
                'required': True,
            },
        },
    }
)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task(iterable=['kind'])
def check(
    ctx,  # type: Context
    target='**/*.py',  # type: str
    recursive=True,  # type: bool
    aggregate=None,  # type: Optional[str]
    number=None,  # type: Optional[str]
    configfile=None,  # type: Optional[str]
    profile=None,  # type: Optional[str]
    tests=None,  # type: Optional[str]
    skip=None,  # type: Optional[str]
    level=False,  # type: bool
    confidence=False,  # type: bool
    kind=None,  # type: Optional[str]
    msg_template=None,  # type: Optional[str]
    output=None,  # type: Optional[str]
    verbose=False,  # type: bool
    debug=False,  # type: bool
    silent=False,  # type: bool
    ignore_nosec=False,  # type: bool
    exclude=None,  # type: Optional[str]
    baseline=None,  # type: Optional[str]
    ini_path=None,  # type: Optional[str]
    exit_zero=False,  # type: bool
):  # type: (...) -> None
    """Perform static code analysis on imports."""
    args = []
    if recursive:
        args.append('--recursive')
    if aggregate:
        args.append(f"--aggregate {aggregate}")
    if number:
        args.append(f"--number {number}")
    if configfile:
        args.append(f"--configfile {configfile}")
    if profile:
        args.append(f"--profile {profile}")
    if tests:
        args.append(f"--tests {tests}")
    if skip:
        args.append(f"--skip {skip}")
    if level:
        args.append('--level')
    if confidence:
        args.append('--confidence')
    if kind:
        args.append(f"--format {','.join(kind)}")
    if msg_template:
        args.append(f"--msg-template {msg_template}")
    if output:
        args.append(f"--output {output}")
    if verbose:
        args.append('--verbose')
    if debug:
        args.append('--debug')
    if silent:
        args.append('--silent')
    if ignore_nosec:
        args.append('--ignore-nosec')
    if exclude:
        args.append(f"--exclude '{','.join(exclude)}'")
    if baseline:
        args.append(f"--baseline {baseline}")
    if ini_path:
        args.append(f"--ini {ini_path}")
    if exit_zero:
        args.append('--exit-zero')
    args.append(target)
    with ctx.cd(ctx.project_dir):
        ctx.run(f"bandit {' '.join(args)}")


namespace = Collection(check)



# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide project initialization."""

import logging
import os
import shutil
from dataclasses import asdict
from typing import TYPE_CHECKING, Any, Dict

import keyring
import pyinputplus as pyip
from invoke import Executor, task

from proman_workflows import WorkflowProgram, __title__, __version__, config
from proman_workflows.collection import Collection
from proman_workflows.config import DocsConfig, ProjectDirs, WorkflowConfig

# TODO: switch to executor
from proman_workflows.pki.gpg import ELYPTICAL_KEY_TYPES, KEY_TYPES

if TYPE_CHECKING:
    from gpg import GenKey
    from invoke import Context

logging.getLogger(__name__).addHandler(logging.NullHandler())


@task
def _setup_executor(ctx):  # type: (Context) -> None
    """Set context."""
    configuration = {
        'plugins': [
            {
                'name': 'vcs',
                'driver_name': 'git',
                'driver_namespace': 'proman.workflows.vcs',
            },
            {
                'name': 'gpg',
                'driver_name': 'gpg',
                'driver_namespace': 'proman.workflows.pki',
            },
        ],
        **asdict(ProjectDirs()),
    }
    length = len(configuration['plugins']) - 1
    if 'plugins' in ctx:
        for i, plugin in enumerate(configuration['plugins']):
            for update in ctx['plugins']:
                if update['name'] == plugin['name']:
                    configuration['plugins'][i].update(plugin)
                elif i == length and update not in configuration['plugins']:
                    configuration['plugins'].append(update)

    setup_tasks = Collection(configuration=configuration)
    ctx.__executor = Executor(setup_tasks)


def create_signingkey(ctx, name, email):  # type: (Context, str, str) -> GenKey
    """Create a signing key."""
    # TODO: get comment
    settings = {
        'name': name,
        'email': email,
        'comment': 'generated by proman_workflows',
        'key_type': 'EDDSA',
        'subkey_type': 'EDDSA',
        'expire_date': '1y',
    }

    rsp = pyip.inputYesNo('Perform adanced GPG setup (y/n): ')
    if rsp == 'yes':
        # TODO: enum auto
        settings['key_type'] = pyip.inputMenu(
            KEY_TYPES,
            prompt='Select GPG key type:\n',
            default=settings['key_type'],
            numbered=True,
        )
        if settings['key_type'] not in ELYPTICAL_KEY_TYPES:
            settings['subkey_length'] = pyip.inputMenu(
                [1024, 2048, 3076, 4096],
                prompt='Select key size:\n',
                default=4096,
            )
        # elif key_type in ELYPTICAL_KEY_TYPES:
        #     if key_curve:
        #         # and key_curve in KEY_CURVES:
        #         settings['key_curve'] = key_curve
        # if key_usage and all(item in key_usage for item in usage):
        #     settings['key_usage'] = ','.join(key_usage)

        settings['subkey_type'] = pyip.inputMenu(
            KEY_TYPES,
            prompt='Select GPG subkey type:\n',
            default=settings['subkey_type'],
            numbered=True,
        )

        # key_length = pyip.inputInt('Enter key length: ')
        settings['expire_date'] = pyip.inputStr(
            prompt='Enter expiration date: ',
            allowRegexes=[r'^(0|([\d]{1,4}[d|w|m|y]))$'],
            default=settings['expire_date'],
        )
    settings['password'] = pyip.inputPassword(
        'Enter GPG password: ', limit=255
    )

    key = ctx.__executor.execute(('gpg.gen_key', settings))

    signingkey = key['keyid']
    if not keyring.get_password(f"{name}-signingkey", signingkey):
        keyring.set_password(
            f"{name}-signingkey", signingkey, settings['password']
        )

    return signingkey


def setup_gitconfig(ctx, update):  # type: (Context, bool) -> Dict[str, Any]
    """Ensure version control system is setup."""
    print('Check git user info is setup.', end='\n\n')
    result = ctx.__executor.execute(('vcs.config.load', {'scope': 'global'}))
    gitconfig = [y for x, y in result.items() if x.name == 'load'][0]

    # select gpg if found else generate gpg key
    if 'user' not in gitconfig['sections']:
        print('Verfying git user is defined...', end=' ')

    # if not config.retrieve('.gpg.signingkey'):
    try:
        print('Verfying git user.name is defined...', end=' ')
        name = gitconfig['sections']['user']['name']
        print('found')
    except Exception as err:
        logging.info(err)
        print('missing', end='\n\n')
        name = pyip.inputStr('Enter git user.name: ', limit=255)
        gitconfig['sections']['user']['name'] = name

    try:
        print('Verfying git user.email is defined...', end=' ')
        email = gitconfig['sections']['user']['email']
        print('found')
    except Exception as err:
        logging.info(err)
        print('missing', end='\n\n')
        email = pyip.inputEmail('Enter git user.email: ', limit=255)
        gitconfig['sections']['user']['email'] = email

    try:
        print('Verfying git user.sigingkey is defined...', end=' ')
        signingkey = gitconfig['sections']['user']['signingkey']
        print('found')
    except Exception as err:
        logging.info(f"{err}: no gpg signingkey defined")
        print('missing', end='\n\n')

        result = ctx.__executor.execute(
            ('gpg.list_keys', {'secret': False, 'keys': None, 'sigs': False})
        )
        gpg_keys = [y for x, y in result.items() if x.name == 'list_keys'][0]

        choices = ['Create a new subkey', 'Skip']
        if gpg_keys != []:
            print(
                'A GPG key was found but user.signingkey is undefined:',
                end='\n\n',
            )
            choices = [
                f"{k['keyid']}:{k['uids'][0]}" for k in gpg_keys
            ] + choices

        selection = pyip.inputMenu(
            choices=choices,
            prompt='Create a key or use an existing one:\n',
            numbered=True,
        )

        signingkey = None
        if selection == 'Create a new subkey':
            signingkey = create_signingkey(ctx, name, email)
        elif selection == 'Skip':
            return gitconfig
        else:
            signingkey = selection.split(':')[0]

        if signingkey:
            gitconfig['sections']['user']['signingkey'] = signingkey

    # setup keyring secrets
    if not hasattr(ctx, 'gpg'):
        ctx.gpg = dict()

    # TODO: setup gpg commit signing
    # TODO: setup gpg package signing

    ctx.__executor.execute(
        (
            'vcs.config.dump',
            {
                'data': gitconfig,
                'template_name': 'gitconfig',
                'dest': os.path.join(os.path.expanduser('~'), '.gitconfig'),
                'update': update,
            },
        )
    )
    return gitconfig


def setup_gitignore(ctx):  # type: (Context) -> None
    """Check gitignore setup."""
    ...


def setup_githooks(ctx):  # type: (Context) -> None
    """Check githooks setup."""
    ...


@task(_setup_executor)
def setup(ctx, update=False):  # type: (Context, bool) -> None
    """Configure workspace for project development."""
    clear = shutil.which('clear') or shutil.which('cls')
    if clear:
        ctx.run(clear)
    print('This tool will assist with environment setup.', end='\n\n')

    # from pprint import pprint
    # pprint(ctx.config.tool.proman.workflows)
    # pprint(ctx.config.__dict__)
    # pprint(ctx.yey)

    setup_gitconfig(ctx, update)
    # setup_gitignore(ctx)
    # setup_githooks(ctx)

    # if not os.path.exists(config.directory):
    #     os.mkdir(ctx.config_dir)

    # if update or not os.path.exists(config.filepath):
    #     config.dump()


project_config = config.ProjectConfig(
    docs=asdict(DocsConfig()),
    plugins=[
        config.Plugin(
            name='vcs',
            driver_name='git',
            driver_namespace='proman.workflows.vcs',
        ),
        config.Plugin(
            name='sort-headers',
            driver_name='isort',
            driver_namespace='proman.workflows.formatter',
        ),
        config.Plugin(
            name='format',
            driver_name='black',
            driver_namespace='proman.workflows.formatter',
        ),
        config.Plugin(
            name='gpg',
            driver_name='gpg',
            driver_namespace='proman.workflows.pki',
        ),
        config.Plugin(
            name='tls',
            driver_name='tls',
            driver_namespace='proman.workflows.pki',
        ),
    ],
)
namespace = Collection(setup)
namespace.configure(asdict(project_config))
namespace.load_collections()

setup = WorkflowProgram(
    name=__title__,
    namespace=namespace,
    binary='workflow-setup',
    binary_names=['workflow-setup'],
    config_class=WorkflowConfig,
    version=__version__,
)

__all__ = ['namespace', 'setup']



"""Provide git management capabilities."""

from typing import TYPE_CHECKING

from invoke import Collection, task
from pygit2 import Repository

from .config import config_tasks

# from .submodule import submodule_tasks
# import pyinputplus as pyip


if TYPE_CHECKING:
    from invoke import Context


@task
def repo(ctx):  # type: (Context) -> Repository
    """Set git as SCM."""
    return Repository(ctx.repo_dir)


namespace = Collection()
namespace.add_collection(config_tasks, 'config')
# namespace.add_collection(submodule_tasks, 'submodule')



# -*- coding: utf-8 -*-
"""Publish executable using briefcase."""

from typing import TYPE_CHECKING, Optional

from invoke import Collection, task

if TYPE_CHECKING:
    from invoke import Context


@task
def build(
    ctx, update=False, docker=True, verbose=False
):  # type: (Context, bool, bool, bool) -> None
    """Build executable."""
    args = ['--no-input']
    if update:
        args.append('--update')
    if not docker or ctx.container.runtime != 'docker':
        args.append('--no-docker')
    if verbose:
        args.append('--verbosity')
    ctx.run(f"poetry build {' '.join(args)}")


@task
def package(
    ctx,  # type: Context
    kind=None,  # type: Optional[str]
    identity=None,  # type: Optional[str]
    sign=True,  # type: bool
    adhoc_sign=False,  # type: bool
    docker=True,  # type: bool
    verbose=False,  # type: bool
):  # type: (...) -> None
    """Create executable package."""
    args = ['--no-input']
    if kind:
        args.append(f"--packaging-format {kind}")
    if identity:
        args.append(f"--identity {identity}")
    if not sign:
        args.append('--no-sign')
    if adhoc_sign:
        args.append('--adhoc-sign')
    if not docker or ctx.container.runtime != 'docker':
        args.append('--no-docker')
    if verbose:
        args.append('--verbosity')


namespace = Collection(build, package)



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Quality Assurance Task-Runner."""

import logging

from invoke import Collection

logging.getLogger(__name__).addHandler(logging.NullHandler())

# Assemble collections for namespace
namespace = Collection()
# namespace.load_collections(
#     plugins=[
#         {
#             'name': 'sast',
#             'driver_name': 'bandit',
#             'driver_namespace': 'proman.workflows.sca',
#         },
#         {
#             'name': 'dependency-scan',
#             'driver_name': 'safety',
#             'driver_namespace': 'proman.workflows.sca',
#         },
#     ]
# )

__all__ = ['namespace']



# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: Apache 2.0, see LICENSE for more details.
"""Build Task-Runner."""

from invoke import Context, task
from passlib.pwd import genword

from .. import filesystem
from ..container import docker
from ..pki import tls


@task
def start(ctx, certs_path='./nginx/certs'):  # type: (Context, str) -> None
    """Start all services."""
    tls.setup(ctx)
    filesystem.mkdir(ctx, certs_path)
    tls.generate(
        ctx,
        name=['spades.local', 'localhost'],
        key=f"{certs_path}/spades.key",
        cert=f"{certs_path}/spades.crt",
    )
    env = {
        'POSTGRESQL_PASSWORD': genword(entropy=56, length=128),
        'REDIS_PASSWORD': genword(entropy=56, length=128),
    }
    docker.start(ctx, files=[], env=env)


@task
def stop(ctx, certs_path='./nginx/certs'):  # type: (Context, str) -> None
    """Stop all services."""
    docker.stop(ctx)
    tls.clean(ctx)
    filesystem.rm(ctx, certs_path)
